@@ -186,12 +186,13 @@ void FoFiType1::parse() {
 	      }
 	    }
 	  }
-	} else {
-	  if (strtok(buf, " \t") &&
-	      (p = strtok(NULL, " \t\n\r")) && !strcmp(p, "def")) {
-	    break;
-	  }
-	}
+	} 
+
+	// Any line that begins with "def" or contains " def"
+	// terminates the encoding array.
+	if (!strcmp (p, "def") || strstr (buf, " def"))
+	  break;
+
 	line = line1;
       }
       //~ check for getinterval/putinterval junk
