@@ -82,10 +82,7 @@ typedef enum /*< flags >*/
   POPPLER_PERMISSIONS_OK_TO_MODIFY = 1 << 1,
   POPPLER_PERMISSIONS_OK_TO_COPY = 1 << 2,
   POPPLER_PERMISSIONS_OK_TO_ADD_NOTES = 1 << 3,
-  POPPLER_PERMISSIONS_FULL = (POPPLER_PERMISSIONS_OK_TO_PRINT
-			      | POPPLER_PERMISSIONS_OK_TO_MODIFY
-			      | POPPLER_PERMISSIONS_OK_TO_COPY
-			      | POPPLER_PERMISSIONS_OK_TO_ADD_NOTES),
+  POPPLER_PERMISSIONS_FULL = (POPPLER_PERMISSIONS_OK_TO_PRINT | POPPLER_PERMISSIONS_OK_TO_MODIFY | POPPLER_PERMISSIONS_OK_TO_COPY | POPPLER_PERMISSIONS_OK_TO_ADD_NOTES)
 
 } PopplerPermissions;
 
