@@ -13,9 +13,11 @@
 #pragma interface
 #endif
 
-#include <stdio.h>
+#include <stdarg.h>
 #include "poppler-config.h"
 
-extern void CDECL error(int pos, char *msg, ...);
+extern void CDECL error(int pos, char *msg, ...) GCC_PRINTF_FORMAT (2, 3);
+
+void setErrorFunction(void (* f)(int , char *, va_list args));
 
 #endif
