@@ -28,6 +28,32 @@ static inline double clip01(double x) {
   return (x < 0) ? 0 : ((x > 1) ? 1 : x);
 }
 
+GBool Matrix::invertTo(Matrix *other)
+{
+  double det;
+
+  det = 1 / (m[0] * m[3] - m[1] * m[2]);
+  other->m[0] = m[3] * det;
+  other->m[1] = -m[1] * det;
+  other->m[2] = -m[2] * det;
+  other->m[3] = m[0] * det;
+  other->m[4] = (m[2] * m[5] - m[3] * m[4]) * det;
+  other->m[5] = (m[1] * m[4] - m[0] * m[5]) * det;
+
+  return gTrue;
+}
+
+void Matrix::transform(double x, double y, double *tx, double *ty)
+{
+  double temp_x, temp_y;
+
+  temp_x = m[0] * x + m[2] * y + m[4];
+  temp_y = m[1] * x + m[3] * y + m[5];
+
+  *tx = temp_x;
+  *ty = temp_y;
+}
+
 //------------------------------------------------------------------------
 
 static char *gfxColorSpaceModeNames[] = {
@@ -125,6 +151,25 @@ char *GfxColorSpace::getColorSpaceModeName(int idx) {
   return gfxColorSpaceModeNames[idx];
 }
 
+void GfxColorSpace::getRGBLine(Guchar *in, unsigned int *out, int length) {
+  int i, j, n;
+  GfxColor color;
+  GfxRGB rgb;
+
+  n = getNComps();
+  for (i = 0; i < length; i++) {
+    
+    for (j = 0; j < n; j++)
+      color.c[j] = in[i * n + j] / 255.0;
+
+    getRGB (&color, &rgb);
+    out[i] =
+      ((int) (rgb.r * 255) << 16) |
+      ((int) (rgb.g * 255) << 8) |
+      ((int) (rgb.b * 255) << 0);
+  }
+}
+
 //------------------------------------------------------------------------
 // GfxDeviceGrayColorSpace
 //------------------------------------------------------------------------
@@ -143,10 +188,22 @@ void GfxDeviceGrayColorSpace::getGray(GfxColor *color, double *gray) {
   *gray = clip01(color->c[0]);
 }
 
+void GfxDeviceGrayColorSpace::getGrayLine(Guchar *in, Guchar *out, int length) {
+  memcpy (out, in, length);
+}
+
 void GfxDeviceGrayColorSpace::getRGB(GfxColor *color, GfxRGB *rgb) {
   rgb->r = rgb->g = rgb->b = clip01(color->c[0]);
 }
 
+void GfxDeviceGrayColorSpace::getRGBLine(Guchar *in, unsigned int *out,
+					 int length) {
+  int i;
+
+  for (i = 0; i < length; i++)
+    out[i] = (in[i] << 16) | (in[i] << 8) | (in[i] << 0);
+}
+
 void GfxDeviceGrayColorSpace::getCMYK(GfxColor *color, GfxCMYK *cmyk) {
   cmyk->c = cmyk->m = cmyk->y = 0;
   cmyk->k = clip01(1 - color->c[0]);
@@ -228,10 +285,22 @@ void GfxCalGrayColorSpace::getGray(GfxColor *color, double *gray) {
   *gray = clip01(color->c[0]);
 }
 
+void GfxCalGrayColorSpace::getGrayLine(Guchar *in, Guchar *out, int length) {
+  memcpy (out, in, length);
+}
+
 void GfxCalGrayColorSpace::getRGB(GfxColor *color, GfxRGB *rgb) {
   rgb->r = rgb->g = rgb->b = clip01(color->c[0]);
 }
 
+void GfxCalGrayColorSpace::getRGBLine(Guchar *in, unsigned int *out,
+				      int length) {
+  int i;
+
+  for (i = 0; i < length; i++)
+    out[i] = (in[i] << 16) | (in[i] << 8) | (in[i] << 0);
+}
+
 void GfxCalGrayColorSpace::getCMYK(GfxColor *color, GfxCMYK *cmyk) {
   cmyk->c = cmyk->m = cmyk->y = 0;
   cmyk->k = clip01(1 - color->c[0]);
@@ -257,12 +326,32 @@ void GfxDeviceRGBColorSpace::getGray(GfxColor *color, double *gray) {
 		 0.114 * color->c[2]);
 }
 
+void GfxDeviceRGBColorSpace::getGrayLine(Guchar *in, Guchar *out, int length) {
+  int i;
+
+  for (i = 0; i < length; i++) {
+    out[i] = 
+      (in[i * 3 + 0] * 19595 + 
+       in[i * 3 + 0] * 38469 + 
+       in[i * 3 + 0] * 7472) / 65536;
+  }
+}
+
 void GfxDeviceRGBColorSpace::getRGB(GfxColor *color, GfxRGB *rgb) {
   rgb->r = clip01(color->c[0]);
   rgb->g = clip01(color->c[1]);
   rgb->b = clip01(color->c[2]);
 }
 
+void GfxDeviceRGBColorSpace::getRGBLine(Guchar *in, unsigned int *out,
+					int length) {
+  Guchar *p;
+  int i;
+
+  for (i = 0, p = in; i < length; i++, p += 3)
+    out[i] = (p[0] << 16) | (p[1] << 8) | (p[2] << 0);
+}
+
 void GfxDeviceRGBColorSpace::getCMYK(GfxColor *color, GfxCMYK *cmyk) {
   double c, m, y, k;
 
@@ -388,12 +477,32 @@ void GfxCalRGBColorSpace::getGray(GfxColor *color, double *gray) {
 		 0.114 * color->c[2]);
 }
 
+void GfxCalRGBColorSpace::getGrayLine(Guchar *in, Guchar *out, int length) {
+  int i;
+
+  for (i = 0; i < length; i++) {
+    out[i] = 
+      (in[i * 3 + 0] * 19595 + 
+       in[i * 3 + 0] * 38469 + 
+       in[i * 3 + 0] * 7472) / 65536;
+  }
+}
+
 void GfxCalRGBColorSpace::getRGB(GfxColor *color, GfxRGB *rgb) {
   rgb->r = clip01(color->c[0]);
   rgb->g = clip01(color->c[1]);
   rgb->b = clip01(color->c[2]);
 }
 
+void GfxCalRGBColorSpace::getRGBLine(Guchar *in, unsigned int *out,
+				     int length) {
+  Guchar *p;
+  int i;
+
+  for (i = 0, p = in; i < length; i++, p += 3)
+    out[i] = (p[0] << 16) | (p[1] << 8) | (p[2] << 0);
+}
+
 void GfxCalRGBColorSpace::getCMYK(GfxColor *color, GfxCMYK *cmyk) {
   double c, m, y, k;
 
@@ -764,6 +873,11 @@ void GfxICCBasedColorSpace::getRGB(GfxColor *color, GfxRGB *rgb) {
   alt->getRGB(color, rgb);
 }
 
+void GfxICCBasedColorSpace::getRGBLine(Guchar *in, unsigned int *out,
+				       int length) {
+  alt->getRGBLine(in, out, length);
+}
+
 void GfxICCBasedColorSpace::getCMYK(GfxColor *color, GfxCMYK *cmyk) {
   alt->getCMYK(color, cmyk);
 }
@@ -914,6 +1028,22 @@ void GfxIndexedColorSpace::getRGB(GfxColor *color, GfxRGB *rgb) {
   base->getRGB(mapColorToBase(color, &color2), rgb);
 }
 
+void GfxIndexedColorSpace::getRGBLine(Guchar *in, unsigned int *out, int length) {
+  GfxColor color2;
+  Guchar *line;
+  int i, j, n;
+
+  n = base->getNComps();
+  line = (Guchar *) gmalloc (length * n);
+  for (i = 0; i < length; i++)
+    for (j = 0; j < n; j++)
+      line[i * n + j] = lookup[in[i] * n + j];
+
+  base->getRGBLine(line, out, length);
+
+  gfree (line);
+}
+
 void GfxIndexedColorSpace::getCMYK(GfxColor *color, GfxCMYK *cmyk) {
   GfxColor color2;
 
@@ -2049,6 +2179,7 @@ GfxImageColorMap::GfxImageColorMap(int bitsA, Object *decode,
     lookup = (double *)gmalloc((maxPixel + 1) * nComps2 * sizeof(double));
     lookup2 = indexedCS->getLookup();
     colorSpace2->getDefaultRanges(x, y, indexHigh);
+    byte_lookup = (Guchar *)gmalloc ((maxPixel + 1) * nComps2);
     for (i = 0; i <= maxPixel; ++i) {
       j = (int)(decodeLow[0] + (i * decodeRange[0]) / maxPixel + 0.5);
       if (j < 0) {
@@ -2057,28 +2188,47 @@ GfxImageColorMap::GfxImageColorMap(int bitsA, Object *decode,
 	j = indexHigh;
       }
       for (k = 0; k < nComps2; ++k) {
-	lookup[i*nComps2 + k] = x[k] + (lookup2[j*nComps2 + k] / 255.0) * y[k];
+	double mapped;
+
+	mapped = x[k] + (lookup2[j*nComps2 + k] / 255.0) * y[k];
+	lookup[i*nComps2 + k] = mapped;
+	byte_lookup[i*nComps2 + k] = (Guchar) (mapped * 255);
       }
     }
+
   } else if (colorSpace->getMode() == csSeparation) {
     sepCS = (GfxSeparationColorSpace *)colorSpace;
     colorSpace2 = sepCS->getAlt();
     nComps2 = colorSpace2->getNComps();
     lookup = (double *)gmalloc((maxPixel + 1) * nComps2 * sizeof(double));
+    byte_lookup = (Guchar *)gmalloc ((maxPixel + 1) * nComps2);
     sepFunc = sepCS->getFunc();
     for (i = 0; i <= maxPixel; ++i) {
       x[0] = decodeLow[0] + (i * decodeRange[0]) / maxPixel;
       sepFunc->transform(x, y);
       for (k = 0; k < nComps2; ++k) {
 	lookup[i*nComps2 + k] = y[k];
+	byte_lookup[i*nComps2 + k] = (Guchar) (y[k] * 255);
       }
     }
   } else {
     lookup = (double *)gmalloc((maxPixel + 1) * nComps * sizeof(double));
+    byte_lookup = (Guchar *)gmalloc ((maxPixel + 1) * nComps);
+
     for (i = 0; i <= maxPixel; ++i) {
       for (k = 0; k < nComps; ++k) {
+	int byte;
+
 	lookup[i*nComps + k] = decodeLow[k] +
 	                         (i * decodeRange[k]) / maxPixel;
+
+	byte = (int) (lookup[i*nComps + k] * 255 + 0.5);
+	if (byte < 0)
+	  byte = 0;
+	else if (byte > 255)
+	  byte = 255;
+	byte_lookup[i * nComps + k] = byte;
+
       }
     }
   }
@@ -2122,6 +2272,7 @@ GfxImageColorMap::GfxImageColorMap(GfxImageColorMap *colorMap) {
 GfxImageColorMap::~GfxImageColorMap() {
   delete colorSpace;
   gfree(lookup);
+  gfree(byte_lookup);
 }
 
 void GfxImageColorMap::getGray(Guchar *x, double *gray) {
@@ -2162,6 +2313,39 @@ void GfxImageColorMap::getRGB(Guchar *x, GfxRGB *rgb) {
   }
 }
 
+void GfxImageColorMap::getRGBLine(Guchar *in, unsigned int *out, int length) {
+  GfxColor color;
+  double *p;
+  int i, j;
+  Guchar *inp, *outp, *tmp_line;
+  GfxColorSpace *base;
+
+  switch (colorSpace->getMode()) {
+  case csIndexed:
+  case csSeparation:
+    tmp_line = (Guchar *) gmalloc (length * nComps2);
+    for (i = 0; i < length; i++) {
+      for (j = 0; j < nComps2; j++) {
+	tmp_line[i * nComps2 + j] = byte_lookup[in[i] * nComps2 + j];
+      }
+    }
+    colorSpace2->getRGBLine(tmp_line, out, length);
+    gfree (tmp_line);
+    break;
+
+  default:
+    inp = in;
+    for (j = 0; j < length; j++)
+      for (i = 0; i < nComps; i++) {
+	*inp = byte_lookup[*inp * nComps + i];
+	inp++;
+      }
+    colorSpace->getRGBLine(in, out, length);
+    break;
+  }
+
+}
+
 void GfxImageColorMap::getCMYK(Guchar *x, GfxCMYK *cmyk) {
   GfxColor color;
   double *p;
