@@ -116,6 +116,9 @@ PDFDoc::PDFDoc(BaseStream *strA, GooString *ownerPassword,
 GBool PDFDoc::setup(GooString *ownerPassword, GooString *userPassword) {
   str->reset();
 
+  // check footer
+  if (!checkFooter()) return gFalse;
+  
   // check header
   checkHeader();
 
@@ -170,6 +173,38 @@ PDFDoc::~PDFDoc() {
   }
 }
 
+
+// Check for a %%EOF at the end of this stream
+GBool PDFDoc::checkFooter() {
+  // we look in the last 1024 chars because Adobe does the same
+  char *eof = new char[1025];
+  int pos = str->getPos();
+  str->setPos(1024, -1);
+  int i, ch;
+  for (i = 0; i < 1024; i++) {
+    ch = str->getChar();
+    if (ch == EOF)
+      break;
+    eof[i] = ch;
+  }
+  eof[i] = '\0';
+
+  bool found = false;
+  for (i = i - 5; i >= 0; i--) {
+    if (strncmp (&eof[i], "%%EOF", 5) == 0) {
+      found = true;
+      break;
+    }
+  }
+  if (!found) {
+    error(-1, "Document does not have mandatory %%EOF ending");
+    errCode = errDamaged;
+  }
+  delete[] eof;
+  str->setPos(pos);
+  return found;
+}
+  
 // Check for a PDF header on this stream.  Skip past some garbage
 // if necessary.
 void PDFDoc::checkHeader() {
