@@ -17,13 +17,19 @@
 #include <stdio.h>
 #include "goo/gtypes.h"
 #include "GfxFont.h"
+#include "GfxState.h"
 #include "OutputDev.h"
 
 class GooString;
 class GooList;
+class Gfx;
 class GfxFont;
 class GfxState;
 class UnicodeMap;
+class TextBlock;
+class TextPage;
+class TextLineFrag;
+class TextSelectionVisitor;
 
 //------------------------------------------------------------------------
 
@@ -50,6 +56,7 @@ private:
 
   friend class TextWord;
   friend class TextPage;
+  friend class TextSelectionPainter;
 };
 
 //------------------------------------------------------------------------
@@ -68,7 +75,7 @@ public:
 
   // Add a character to the word.
   void addChar(GfxState *state, double x, double y,
-	       double dx, double dy, Unicode u);
+	       double dx, double dy, CharCode c, Unicode u);
 
   // Merge <word> onto the end of <this>.
   void merge(TextWord *word);
@@ -83,6 +90,9 @@ public:
 
   static int cmpYX(const void *p1, const void *p2);
 
+  void visitSelection(TextSelectionVisitor *visitor,
+		      PDFRectangle *selection);
+
 #if TEXTOUT_WORD_LIST
   int getLength() { return len; }
   Unicode getChar(int idx) { return text[idx]; }
@@ -104,6 +114,7 @@ private:
   double yMin, yMax;		// bounding box y coordinates
   double base;			// baseline x or y coordinate
   Unicode *text;		// the text
+  CharCode *charcode;		// glyph indices
   double *edge;			// "near" edge x or y coord of each char
 				//   (plus one extra entry for the last char)
   int len;			// length of text and edge arrays
@@ -129,6 +140,9 @@ private:
   friend class TextFlow;
   friend class TextWordList;
   friend class TextPage;
+
+  friend class TextSelectionPainter;
+  friend class TextSelectionDumper;
 };
 
 //------------------------------------------------------------------------
@@ -161,6 +175,8 @@ private:
   friend class TextPage;
 };
 
+struct TextFlowData;
+
 //------------------------------------------------------------------------
 // TextLine
 //------------------------------------------------------------------------
@@ -192,6 +208,9 @@ public:
 
   void coalesce(UnicodeMap *uMap);
 
+  void visitSelection(TextSelectionVisitor *visitor,
+		      PDFRectangle *selection);
+
 private:
 
   TextBlock *blk;		// parent block
@@ -216,6 +235,10 @@ private:
   friend class TextFlow;
   friend class TextWordList;
   friend class TextPage;
+
+  friend class TextSelectionPainter;
+  friend class TextSelectionSizer;
+  friend class TextSelectionDumper;
 };
 
 //------------------------------------------------------------------------
@@ -247,6 +270,9 @@ public:
   // primary rotation.
   GBool isBelow(TextBlock *blk);
 
+  void visitSelection(TextSelectionVisitor *visitor,
+		      PDFRectangle *selection);
+
 private:
 
   TextPage *page;		// the parent page
@@ -272,6 +298,7 @@ private:
   friend class TextFlow;
   friend class TextWordList;
   friend class TextPage;
+  friend class TextSelectionPainter;
 };
 
 //------------------------------------------------------------------------
@@ -391,7 +418,20 @@ public:
 
   // Get the text which is inside the specified rectangle.
   GooString *getText(double xMin, double yMin,
-		   double xMax, double yMax);
+		     double xMax, double yMax);
+
+  void visitSelection(TextSelectionVisitor *visitor,
+		      PDFRectangle *selection);
+
+  void drawSelection(OutputDev *out,
+		     double scale,
+		     int rotation,
+		     PDFRectangle *selection,
+		     GfxColor *glyph_color, GfxColor *box_color);
+
+  GooList *getSelectionRegion(PDFRectangle *selection, double scale);
+
+  GooString *getSelectionText(PDFRectangle *selection);
 
   // Find a string by character position and length.  If found, sets
   // the text bounding rectangle and returns true; otherwise returns
@@ -454,6 +494,8 @@ private:
   friend class TextBlock;
   friend class TextFlow;
   friend class TextWordList;
+  friend class TextSelectionPainter;
+  friend class TextSelectionDumper;
 };
 
 //------------------------------------------------------------------------
@@ -545,6 +587,14 @@ public:
 		      double *xMin, double *yMin,
 		      double *xMax, double *yMax);
 
+  void drawSelection(OutputDev *out, double scale, int rotation,
+		     PDFRectangle *selection,
+		     GfxColor *glyph_color, GfxColor *box_color);
+
+  GooList *getSelectionRegion(PDFRectangle *selection, double scale);
+
+  GooString *getSelectionText(PDFRectangle *selection);
+
 #if TEXTOUT_WORD_LIST
   // Build a flat word list, in content stream order (if
   // this->rawOrder is true), physical layout order (if
