@@ -150,7 +150,7 @@ public:
     UseOC
   };
   
-  static Document *Document::load(const QString & filePath);
+  static Document *load(const QString & filePath);
   
   Page *getPage(int index) const{ return new Page(this, index); }
   
@@ -172,11 +172,11 @@ public:
   bool okToAddNotes() const;
   double getPDFVersion() const;
   
-  Document::~Document();
+  ~Document();
   
 private:
   DocumentData *data;
-  Document::Document(DocumentData *dataA);
+  Document(DocumentData *dataA);
 };
 
 }
